@@ -16,16 +16,11 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 """
-    sphinx.domains.cl
-    ~~~~~~~~~~~~~~~~~
+    sphinxcontrib.cldomain
+    ~~~~~~~~~~~~~~~~~~~~~~
 
     The Common Lisp domain
 
-
-    TODO
-    ----
-    add source-read handler
-    http://sphinx.pocoo.org/latest/ext/appapi.html#event-source-read
 """
 import re
 import os
@@ -54,13 +49,21 @@ ALL_TYPES = ["macro", "function", "genericFunction",
              "setf", "variable", "type"]
 upper_symbols = re.compile("([^a-z\s\"`]*[A-Z]{2,}[^a-z\s\"`:]*)($|\s)")
 
-DOC_STRINGS = defaultdict(dict)
-TYPES = defaultdict(dict)
-ARGS = defaultdict(dict)
-METHODS = defaultdict(dict)
-SLOTS = defaultdict(dict)
+DOC_STRINGS = defaultdict(dict, {})
+TYPES = defaultdict(dict, {})
+ARGS = defaultdict(dict, {})
+METHODS = defaultdict(dict, {})
+SLOTS = defaultdict(dict, {})
+USED_SYMBOLS = defaultdict(dict, {})
 
-lambda_list_keywords = ["&allow-other-keys", "&key", "&rest", "&aux", "&optional"]
+lambda_list_keywords = ["&allow-other-keys", "&key",
+                        "&rest", "&aux", "&optional"]
+
+
+def record_use(package, symbol_name, objtype):
+    """record unused package symbols."""
+    symbol = symbol_name.upper()
+    USED_SYMBOLS[package].setdefault(symbol, []).append(objtype)
 
 
 def bool_option(arg):
@@ -119,7 +122,7 @@ def resolve_string(state_machine, package, symbol, objtype, specializer=None):
     """
     if objtype == "method":
         spec = specializer[0].split(" ")[1:]
-        method_doc = METHODS.get(package).get(symbol, {})
+        method_doc = METHODS[package].get(symbol, {})
         key = tuple([parse_specializer_symbol(sym, package)
                      for sym in spec])
         if key not in method_doc:
@@ -127,7 +130,7 @@ def resolve_string(state_machine, package, symbol, objtype, specializer=None):
                                            (package, symbol, spec))
         return method_doc.get(key, "")
 
-    possible_strings = DOC_STRINGS.get(package)[symbol]
+    possible_strings = DOC_STRINGS[package][symbol]
 
     # XXX This isn't the best, the objtype is generic but the
     # docstring will be under genericFunction because of the JSON
@@ -357,6 +360,7 @@ class CLsExp(ObjectDescription):
         symbol_name = sig
         if not symbol_name:
             raise Exception("Unknown symbol type for signature %s" % sig)
+        record_use(package, symbol_name, self.objtype)
         return objtype.strip(), symbol_name.upper()
 
     def get_index_text(self, name, type):
@@ -435,7 +439,7 @@ class CLsExp(ObjectDescription):
         package = self.env.temp_data.get('cl:package')
         name = self.names[0][1]
         description = result[1][-1]
-        specializers = METHODS.get(package, {}).get(name).keys()
+        specializers = METHODS[package].get(name, {}).keys()
         if specializers:
             description.append(nodes.paragraph(text="Specializes"))
             spec = nodes.bullet_list()
@@ -568,6 +572,7 @@ def code_regions(text):
     indent = False
     for line in io:
         if indent is False and (line.startswith(" ") or line.startswith("\t")):
+            # TODO detect if its a REPL.
             output.write("\n.. code-block:: common-lisp\n\n")
             indent = True
         if indent is True and not (line.startswith(" ") or line.startswith("\t")):
@@ -593,9 +598,6 @@ def index_package(package, package_path, quicklisp):
                         if not line.startswith(";")])
 
     lisp_data = json.loads(output)
-    DOC_STRINGS[package] = {}
-    METHODS[package] = {}
-    SLOTS[package] = {}
     for k, v in lisp_data.items():
 
         # extract doc strings
@@ -631,8 +633,6 @@ def index_package(package, package_path, quicklisp):
         if "slots" in v:
             SLOTS[package][k] = v["slots"]
 
-    ARGS[package] = {}
-
     def lower_symbols(text):
         if '"' in text:
             return text
@@ -671,6 +671,25 @@ def uppercase_symbols(app, docname, source):
                            ":cl:symbol:`~\g<1>`\g<2>", line)
 
 
+def list_unused_symbols(app, exception):
+    if exception:
+        return
+    # TODO (RS) this initial implementation will not be able to detect
+    # if each method specialisation has been used.
+    for p, sym_doc in DOC_STRINGS.items():
+        for s, docs in sym_doc.items():
+            for objtype in docs.keys():
+                if s in USED_SYMBOLS[p]:
+                    if objtype == "genericFunction":
+                        objtype = "generic"
+                    if objtype not in USED_SYMBOLS[p][s]:
+                        app.warn("Unused symbol doc %s:%s type %s" %
+                                           (p, s, objtype))
+                else:
+                    app.warn("Unused symbol doc %s:%s type %s" %
+                                           (p, s, objtype))
+
+
 def setup(app):
     app.add_domain(CLDomain)
     app.add_node(desc_clparameterlist,
@@ -687,6 +706,7 @@ def setup(app):
     app.add_config_value('cl_quicklisp', "", 'env')
     app.add_config_value('cl_show_defaults', False, True)
     app.connect('builder-inited', load_packages)
+    app.connect('build-finished', list_unused_symbols)
     #app.connect('source-read', uppercase_symbols)
 
 
